"""
empty message

Revision ID: 1f4631da751b
Revises: 946bada3d9d5
Create Date: 2017-05-24 20:19:54.896355
"""

# revision identifiers, used by Alembic.
from __future__ import annotations

revision = "1f4631da751b"
down_revision = "946bada3d9d5"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_foreign_key(
        "fk_inherited_entity_id", "sbe_event", "entity", ["id"], ["id"], use_alter=True
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("fk_inherited_entity_id", "sbe_event", type_="foreignkey")
    # ### end Alembic commands ###
