# Copyright (c) 2012-2024, Abilian SAS

""""""

from __future__ import annotations

from .models import Vocabulary
from .service import get_vocabulary, vocabularies

__all__ = ["Vocabulary", "get_vocabulary", "vocabularies"]
